@@ -60,22 +60,25 @@ class ProofServer(socketserver.ThreadingUDPServer):
         status_path: Path,
         **kwargs: Any,
     ) -> None:
+        self.status = Status(status_path)
+        if not self.status.touch():
+            raise ServerError("Status file already exists or couldn't be created.")
+
         super().__init__((consts.HOST, port), ProofHandler)
         level = {0: logging.NOTSET, 1: logging.INFO}.get(debug, logging.DEBUG)
         self.logger = self.init_logger(level, log_path)
         self.queue: Queue[str] = Queue(maxsize=nprefetch)
         self.limit = limit
+
+        host, port = self.server_address
+        if not self.status.write(pid=os.getpid(), host=host, port=port):
+            self.status.remove()
+            raise ServerError("Failed to write status file.")
+
         threading.Thread(
             target=self.fetch_proofs, daemon=True, name="ServerLoop"
         ).start()
 
-        self.pid = os.getpid()
-        self.status = Status(status_path)
-        host, port = self.server_address
-        if not self.status.write(pid=self.pid, host=host, port=port):
-            self.status.remove()
-            raise ServerError("Failed to write status file.")
-
     def init_logger(self, level: int, path: Path) -> logging.Logger:
         logger = logging.getLogger(__name__)
         logger.setLevel(level)
@@ -95,7 +98,7 @@ class ProofServer(socketserver.ThreadingUDPServer):
     def server_close(self) -> None:
         super().server_close()
         status = self.status.read()
-        if status is not None and status["pid"] == self.pid:
+        if status is not None and status["pid"] == os.getpid():
             self.status.remove()
 
     def fetch_proof(self, name: Optional[str] = None) -> Optional[str]:
@@ -148,8 +151,11 @@ def spawn(**kwargs: Any) -> None:
             signal.SIGTERM,
             lambda signum, frame: threading.Thread(target=server.shutdown).start(),
         )
-        with ProofServer(**kwargs) as server:
-            server.serve_forever()
+        try:
+            with ProofServer(**kwargs) as server:
+                server.serve_forever()
+        except ServerError as e:
+            sys.exit(str(e))
 
 
 def server_start(status: Status, force: bool, **kwargs: Any) -> None:
@@ -170,8 +176,10 @@ def server_stop(status: Status) -> None:
         raise ServerError("Failed to stop daemon.")
 
 
-def server_status(status: Status) -> str:
+def server_status(status: Status, wait: bool) -> str:
     try:
+        if wait:
+            status.wait(exist=True, timeout=None)
         return str(status)
     except ValueError:
         raise ServerError("Failed to read status file.")
@@ -200,6 +208,7 @@ def main() -> None:
     parser.add_argument("-p", "--port", type=int, default=0)
     parser.add_argument("-f", "--force", action="store_true")
     parser.add_argument("-q", "--quiet", action="store_true")
+    parser.add_argument("-w", "--wait", action="store_true")
     args = parser.parse_args()
 
     if args.quiet:
@@ -215,7 +224,7 @@ def main() -> None:
             server_stop(status)
             server_start(status, **vars(args))
         elif args.action == "status":
-            print(server_status(status))
+            print(server_status(status, args.wait))
         else:
             sys.exit(f"Unrecognized action: {args.action}.")
     except ServerError as e:
